%Point matching test algorithm 
%Longin Jan Latecki, latecki@temple.edu, November 2012 

%the toy example for matching 7 points to 6 points called labels
%A is matrix of pairwise affinites of pairs (point, label)
%M is mutex matrix on pairs (point, label)

function [matchingaccuracy, time, points, labels]=pointmatching

close all
clear all
labels =[3 11;2 11; 1 11;1 10;1 9;1 8;1 7; 2 7;3 7;4 7];
labels =[labels; 7 10;7 9;7 8;7 7];
labels =[labels; 12 11;11 11;10 11;10 10;10 9;11 9;12 9;13 9;13 8;13 7;12 7;11 7;10 7];
labels = 2*labels;
[size_l,~]=size(labels);
trueCorr=[1:size_l];

% generate points
points = labels;

% add more points/duplicate points
%  points = [points; 8 24; 8 24; 20 24; 20 24; 24 24; 28 24; 16 12; 16 12];
%  trueCorr = [trueCorr 1 1 6 6 8 10 16 16];
%  [trueCorr, new_idx] = sort(trueCorr,2); % sort trueCorr
%  points=points(new_idx,:);  % reorder points with the index of trueCorr


% add outliers
points = [points; 14 26];
trueCorr = [trueCorr 0];

% randomize order
[~, size_r] = size(trueCorr);
random_order = randperm(size_r);
trueCorr = trueCorr(random_order);
points = points(random_order',:);


% add noise
points = points + 0.3*randn(size(points)); %points plus noise


% rotation (counterclockwise)
%   shift to origin (0,0)
mean_point=mean(points);
points(:,1)=points(:,1)-mean_point(1);
points(:,2)=points(:,2)-mean_point(2);
%   rotate about origin (counterclockwise)
%theta = deg2rad(-75);  % rotation angle
theta = deg2rad(rand*360);  % rotation angle
rot = [cos(theta) -sin(theta); sin(theta) cos(theta)];  % rotation matrix
points = points*rot';
%   shift back
points(:,1)=points(:,1)+mean_point(1);
points(:,2)=points(:,2)+mean_point(2);


% translation/shifting all points
points(:,1) = points(:,1) + randn*20;   % along x
points(:,2) = points(:,2) + randn*20;   % along y



npoints=length(points);
nlabels=length(labels);
%shows the configuration of points in blue and labels in red
figure; plot(points(:,1),points(:,2),'k*','MarkerSize',14);
for i=1:length(points)
    text(points(i,1)+0.05,points(i,2)+0.1,num2str(trueCorr(1,i)));
end
hold on; axis equal;
plot(labels(:,1),labels(:,2),'r^','MarkerSize',14); 
for i=1:length(labels)
    text(labels(i,1)+0.05,labels(i,2)+0.1,num2str(i));
end
title('point configuration');



%calling the matching function
t1 = tic;
[C]=pointmatchfun(points,labels,1); % at most 1 points for a label
time=toc(t1)

%computes the matching accuracy
matchingaccuracy= sum(C==trueCorr)/length(trueCorr)



% reverse the permutation
reverse_permutation = zeros(1,size_r);
for i=1:size_r
    reverse_permutation(1,random_order(1,i))=i;
end
C=C(reverse_permutation);

 %vizualization
CorrProb=zeros(npoints,nlabels);
for i=1:length(C)
    if C(i) <= nlabels && C(i) > 0
        CorrProb(i,C(i))=1;
    end
end
figure; imagesc(CorrProb); title('corr by pointmatchfun');

end



